@@ -51,6 +51,7 @@ class XmlRfcDummy():
             "linkmailto":	"yes",
             #"linefile":	Unset
             #"needLines":       Unset
+            "multiple-initials":"no",
             "notedraftinprogress": "yes",
             "private":		"",
             "refparent":	"References",
@@ -172,6 +173,18 @@ class WriterElementTest(unittest.TestCase):
     def test_texttable_small(self):
         return self.function_test("texttable_small", "draw_table")
 
+    def test_texttable_small_full(self):
+        return self.function_test("texttable_small_full", "draw_table")
+
+    def test_texttable_small_all(self):
+        return self.function_test("texttable_small_all", "draw_table")
+
+    def test_texttable_small_headers(self):
+        return self.function_test("texttable_small_headers", "draw_table")
+
+    def test_texttable_small_none(self):
+        return self.function_test("texttable_small_none", "draw_table")
+
     def test_figure_title(self):
         return self.function_test("figure_title", "write_t_rec")
 
@@ -370,4 +383,4 @@ class WriterRfcTest(WriterRootTest):
         return self.status_test()
 
 if __name__ == '__main__':
-    unittest.main(verbosity=2)
+    unittest.main()
