@@ -1,4 +1,5 @@
 
+from xml2rfc.writers.base import RfcWriterError
 from xml2rfc.writers.base import BaseRfcWriter
 from xml2rfc.writers.raw_txt import RawTextRfcWriter
 from xml2rfc.writers.paginated_txt import PaginatedTextRfcWriter
@@ -7,4 +8,5 @@ from xml2rfc.writers.nroff import NroffRfcWriter
 from xml2rfc.writers.expanded_xml import ExpandedXmlWriter
 
 __all__ = ['BaseRfcWriter', 'RawTextRfcWriter', 'PaginatedTextRfcWriter',
-           'HtmlRfcWriter', 'NroffRfcWriter', 'ExpandedXmlWriter',]
+           'HtmlRfcWriter', 'NroffRfcWriter', 'ExpandedXmlWriter',
+           'RfcWriterError',]
