@@ -2,15 +2,14 @@
 # Copyright The IETF Trust 2011, All Rights Reserved
 # --------------------------------------------------
 
-import copy
 import codecs
-import string
 import datetime
 import lxml
 import xml2rfc.log
 import xml2rfc.utils
 try:
     import debug
+    assert debug
 except ImportError:
     pass
 
@@ -408,7 +407,8 @@ class BaseRfcWriter:
         initials = author.attrib.get('initials', '')
         initials_list = re.split("[. ]+", initials)
         if self.pis["multiple-initials"] == "no":
-            initials = initials_list[0] + "."
+            if not initials_list[0] == "":
+                initials = initials_list[0] + "."
         else:
             initials = ". ".join(initials_list) + "."
         return initials
@@ -437,15 +437,27 @@ class BaseRfcWriter:
         if date is not None:
             year = date.attrib.get('year', '')
             month = date.attrib.get('month', '')
-            # If year is this year, and month not specified, use current date
-            if not year or (year == str(today.year) and not month) or \
-                           (self.draft and year == str(today.year) and month == today.strftime("%B")):
+            day = date.attrib.get('day', '')
+            if not year or (year == str(today.year) and\
+               (not month or month.lower() == today.strftime("%B").lower() or \
+                month.lower() == today.strftime("%b").lower())):
                 # Set everything to today
                 date.attrib['year'] = today.strftime('%Y')
                 date.attrib['month'] = today.strftime('%B')
-                date.attrib['day'] = today.strftime('%d')
+                if self.draft and not day:
+                    date.attrib['day'] = today.strftime('%d')
+                    if date.attrib['day'][0] == '0':
+                        date.attrib['day'] = today.strftime('%d').replace('0', '')
             elif year != str(today.year) and not month:
                 xml2rfc.log.warn("Incomplete and out-of date <date/> element: %s" % lxml.etree.tostring(date))
+        try:
+            datetime.datetime.strptime(date.attrib.get('year')+date.attrib.get('month'), '%Y%B')
+        except ValueError:
+            try:
+                datetime.datetime.strptime(date.attrib.get('year')+date.attrib.get('month'), '%Y%b')
+            except ValueError:
+                xml2rfc.log.warn("Year and/or month are incorrect values in <date/> element: %s" % lxml.etree.tostring(date))
+
         # Setup the expiration string for drafts as published date + six months
         if self.draft:
             date = self.r.find('front/date')
@@ -453,17 +465,19 @@ class BaseRfcWriter:
                 month = date.attrib.get('month', '')
                 year = date.attrib.get('year', '')
                 day = date.attrib.get('day', '1')  # Default to first of month
-                if month and year:
+                try:
+                    start_date = datetime.datetime.strptime(year + month + day, \
+                                                            '%Y%B%d')
+                except ValueError:
                     try:
                         start_date = datetime.datetime.strptime(year + month + day, \
-                                                                '%Y%B%d')
-                        expire_date = start_date + datetime.timedelta(185)
-                        self.expire_string = expire_date.strftime('%B %d, %Y')
+                                                                '%Y%b%d')
                     except ValueError:
-                        pass
-                elif not year:
-                    # Warn about no date
-                    xml2rfc.log.warn('No date specified for document.')
+                        start_date = today
+
+                expire_date = start_date + datetime.timedelta(185)
+                self.expire_string = expire_date.strftime('%B %d, %Y').replace(' 0', ' ')
+
 
     def _format_counter(self, text, count, list_length=1):
         """ Return a proper string for a formatted list bullet.  Allowed types:
@@ -472,6 +486,9 @@ class BaseRfcWriter:
                 %d: Digits
                 %i: Lowercase roman numerals
                 %I: Uppercase roman numerals
+                %o: octal
+                %x: Lowercase hex
+                %X: Uppercase hex
         """
         import math
         roman_widths = {        1:1,  2:2,  3:3,  4:2,  5:1,  6:2,  7:3,  8:4,  9:2,
@@ -481,6 +498,8 @@ class BaseRfcWriter:
         decimal_width = int(math.log(list_length, 10))
         roman_width = roman_widths.get(list_length, 6)
         letter_width = int(math.log(list_length, 26))
+        hex_width = int(math.log(list_length, 16))
+        octal_width = int(math.log(list_length, 8))
         extra_width = len(text)+1
         if '%d' in text:
             text = text.replace(r'%d', str(count)).ljust(decimal_width+extra_width)
@@ -492,6 +511,12 @@ class BaseRfcWriter:
             text = text.replace(r'%i', xml2rfc.utils.int2roman(count)).ljust(roman_width+extra_width)
         elif '%I' in text:
             text = text.replace(r'%I', xml2rfc.utils.int2roman(count).upper()).ljust(roman_width+extra_width)
+        elif '%o' in text:
+            text = text.replace(r'%o', oct(count).replace("0","",1)).replace("o","",1).ljust(octal_width+extra_width)
+        elif '%x' in text:
+            text = text.replace(r'%x', hex(count).replace("0x","",1)).ljust(hex_width+extra_width)
+        elif '%X' in text:
+            text = text.replace(r'%X', hex(count).replace("0x","",1).upper()).ljust(hex_width+extra_width)
         return text
 
     def _format_author_string(self, authors):
@@ -500,11 +525,13 @@ class BaseRfcWriter:
         for i, author in enumerate(authors):
             organization = author.find('organization')
             surname = author.attrib.get('surname', '')
+            if i == len(authors) - 1 and len(authors) > 1:
+                buf.append('and ')
             if surname:
                 initials = self.get_initials(author)
                 if i == len(authors) - 1 and len(authors) > 1:
                     # Last author is rendered in reverse
-                    buf.append('and ' + initials + ' ' + \
+                    buf.append(initials + ' ' + \
                                      surname)
                 else:
                     buf.append(surname + ', ' + initials)
@@ -512,7 +539,7 @@ class BaseRfcWriter:
                     buf.append(', Ed.')
             elif organization is not None and organization.text:
                 # Use organization instead of name
-                buf.append(organization.text)
+                buf.append(organization.text.strip())
             if len(authors) == 2 and i == 0:
                 buf.append(' ')
             elif i < len(authors) - 1:
@@ -577,7 +604,7 @@ class BaseRfcWriter:
             lines.append('ISSN: %s' % self.boilerplate['issn'])
 
         # Strip any whitespace from XML to make header as neat as possible
-        lines = map(string.strip, lines)
+        lines = [ l.strip() for l in lines ]
         return lines
 
     def _prepare_top_right(self):
@@ -625,7 +652,7 @@ class BaseRfcWriter:
                 day = day + ', '
             lines.append(month + day + year)
             # Strip any whitespace from XML to make header as neat as possible
-            lines = map(string.strip, lines)
+            lines = [ l.strip() for l in lines ]
         return lines
 
     def write_figure(self, figure):
@@ -653,12 +680,12 @@ class BaseRfcWriter:
 
         # Write figure with optional delimiter
         delimiter = self.pis['artworkdelimiter']
-        artwork = figure.find('artwork')
+        artwork = figure.find('artwork').text
         # artwork_align = artwork.attrib.get('align', align)
         # Explicitly use figure alignment
         artwork_align = align
         blanklines = int(self.pis['artworklines'])
-        self.write_raw(figure.find('artwork').text, align=artwork_align,
+        self.write_raw(artwork, align=artwork_align,
                        blanklines=blanklines, delimiter=delimiter,
                        source_line=figure.sourceline)
 
@@ -675,7 +702,8 @@ class BaseRfcWriter:
             self.write_label('Figure ' + str(self.figure_count) + title,
                             type='figure', source_line=figure.sourceline)
         else:
-            self.write_label(title, type='figure', source_line=figure.sourceline)
+            if title:
+                self.write_label(title, type='figure', source_line=figure.sourceline)
 
     def write_table(self, table):
         """ Writes <texttable> elements """
@@ -716,7 +744,8 @@ class BaseRfcWriter:
             self.write_label('Table ' + str(self.table_count) + title, \
                              type='table', source_line=table.sourceline)
         else:
-            self.write_label(title, type='table', source_line=table.sourceline)
+            if title:
+                self.write_label(title, type='table', source_line=table.sourceline)
 
     def _index_t_rec(self, element):
         """ Traverse a <t> element only performing indexing operations """
@@ -780,7 +809,8 @@ class BaseRfcWriter:
         for child_sec in section.findall('section'):
             if appendix == True and not count_str:
                 # Use an alphabetic counter for first-level appendix
-                self.write_section_rec(child_sec, string.uppercase[s_count - 1],
+                uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                self.write_section_rec(child_sec, uppercase[s_count - 1],
                                         level=level + 1, appendix=True)
             else:
                 # Use a numeric counter
@@ -934,9 +964,10 @@ class BaseRfcWriter:
                                       subCounter=i+1, level=2)
         for reference_list in references:
             for ref in reference_list:
-                ref_counter += 1
-                title = ref.find("front/title").text
-                self._indexRef(ref_counter, title=title, anchor=ref.attrib["anchor"])
+                if len(ref):
+                    ref_counter += 1
+                    title = ref.find("front/title").text
+                    self._indexRef(ref_counter, title=title, anchor=ref.attrib["anchor"])
 
         # Appendix sections
         back = self.r.find('back')
