@@ -12,6 +12,7 @@ import datetime
 import cgi
 try:
     import debug
+    assert debug
 except ImportError:
     pass
 
@@ -46,7 +47,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                          'address_card.html']:
             file = open(os.path.join(self.templates_dir, filename), 'r')
             self.templates[filename] = string.Template(file.read())
-
+            file.close()
         # Buffers to aggregate various elements before processing template
         self.buffers = {'front': [],
                         'body': [],
@@ -66,8 +67,18 @@ class HtmlRfcWriter(BaseRfcWriter):
         # Table to insert the iref into
         self.iref_table = None
 
-    def write_list(self, list, parent):
-        style = list.attrib.get('style', 'empty')
+    def write_list(self, list, parent, level=0):
+        # style comes from the node if one exists
+        style = list.attrib.get('style', '')
+        if not style:
+            # otherwise look for the nearest list parent with a style and use it
+            for elder in list.iterancestors():
+                if elder.tag == 'list':
+                    style = elder.attrib.get('style', '')
+                    if style:
+                        break
+        if not style:
+            style = 'empty'
         if style == 'hanging':
             list_elem = E.DL()
             hangIndent = list.attrib.get('hangIndent', '8')
@@ -78,11 +89,11 @@ class HtmlRfcWriter(BaseRfcWriter):
                 dd = E.DD(style=style)
                 list_elem.append(dt)
                 list_elem.append(dd)
-                self.write_t_rec(t, parent=dd)
+                self.write_t_rec(t, parent=dd, level=level +1)
         elif style.startswith('format'):
             format_str = style.partition('format ')[2]
-            allowed_formats = ('%c', '%C', '%d', '%i', '%I')
-            if not any(map(lambda format: format in format_str, allowed_formats)):
+            allowed_formats = ('%c', '%C', '%d', '%i', '%I', '%o', '%x', '%X')
+            if not any([ f in format_str for f in allowed_formats]):
                 xml2rfc.log.warn('Invalid format specified: %s ' 
                                  '(Must be one of %s)' % (style,
                                     ', '.join(allowed_formats)))
@@ -102,26 +113,28 @@ class HtmlRfcWriter(BaseRfcWriter):
                 dd = E.DD()
                 list_elem.append(dt)
                 list_elem.append(dd)
-                self.write_t_rec(t, parent=dd)
+                self.write_t_rec(t, parent=dd, level=level + 1)
         else:
             if style == 'symbols':
                 list_elem = E.UL()
             elif style == 'numbers':
                 list_elem = E.OL()
             elif style == 'letters':
-                list_elem = E.OL(style="list-style-type: lower-alpha")
+                letter_style = "list-style-type: upper-alpha" if (level % 2) else "list-style-type: lower-alpha"
+                list_elem = E.OL(style= letter_style)
             else:  # style == empty
                 list_elem = E.UL()
                 list_elem.attrib['class'] = 'empty'
             for t in list.findall('t'):
                 li = E.LI()
                 list_elem.append(li)
-                self.write_t_rec(t, parent=li)
+                self.write_t_rec(t, parent=li, level= level + 1)
         parent.append(list_elem)
 
     def _create_toc(self):
         # Add link for toc itself
-        link = E.LINK(rel='Contents', href='#rfc.toc')
+        link = E.LINK(href='#rfc.toc')
+        link.attrib["rel"] = 'Contents'
         self.buffers['toc_head_links'].append(self._serialize(link))
         tocdepth = self.pis['tocdepth']
         try:
@@ -149,7 +162,10 @@ class HtmlRfcWriter(BaseRfcWriter):
                 li.append(a)
                 self.buffers['toc_rows'].append(self._serialize(li))
     def _serialize(self, element):
-        return lxml.html.tostring(element, pretty_print=True, method='xml')
+        if sys.version > '3':
+            return lxml.html.tostring(element, pretty_print=True, method='xml', encoding='ascii').decode()
+        else:
+            return lxml.html.tostring(element, pretty_print=True, method='xml')
     
     def _flush_temp_div(self):
         lines = []
@@ -282,7 +298,8 @@ class HtmlRfcWriter(BaseRfcWriter):
             h.append(a_bullet)
             if anchor:
                 # Use an anchor link for heading
-                a_text = E.A(text, href='#' + anchor, id=anchor)
+                a_text = E.A(text, href='#' + anchor, )
+                a_text.attrib["id"] = anchor
                 h.append(a_text)
             else:
                 # Plain text
@@ -305,7 +322,7 @@ class HtmlRfcWriter(BaseRfcWriter):
             # Add to body buffer
             self.buf.append(self._serialize(p))
 
-    def write_t_rec(self, t, autoAnchor=None, align='left', parent=None):
+    def write_t_rec(self, t, autoAnchor=None, align='left', parent=None, level=0):
         """ Recursively writes a <t> element
 
             If no parent is specified, a dummy div will be treated as the parent
@@ -343,7 +360,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                 if child.tail:
                     br.tail = child.tail
             elif child.tag == 'list':
-                self.write_list(child, parent)
+                self.write_list(child, parent, level=level)
                 if child.tail:
                     parent.append(E.P(child.tail))
             elif child.tag == 'figure':
@@ -475,10 +492,11 @@ class HtmlRfcWriter(BaseRfcWriter):
                 email = author.find('address/email')
                 surname = author.attrib.get('surname')
                 initials = self.get_initials(author)
+                if j == len(authors) - 1 and len(authors) > 1:
+                    last.tail = ' and '
                 if surname is not None:
                     if j == len(authors) - 1 and len(authors) > 1:
                         # Last author, render in reverse
-                        last.tail = ' and '
                         name_string = initials + ' ' + surname 
                     else:
                         name_string = surname + ', ' + initials
@@ -487,19 +505,19 @@ class HtmlRfcWriter(BaseRfcWriter):
                         if self.pis['linkmailto'] == 'yes':
                             a.attrib['href'] = 'mailto:' + email.text
                     if organization is not None and organization.text:
-                        a.attrib['title'] = organization.text
+                        a.attrib['title'] = organization.text.strip()
                     ref_td.append(a)
                     last = a
-                    a.tail = ', '
                 elif organization is not None and organization.text:
                     # Use organization instead of name
-                    a = E.A(organization.text)
+                    a = E.A(organization.text.strip())
                     ref_td.append(a)
                     last = a
+                a.tail = ', '
             last.tail = ', "'
             title = reference.find('front/title')
             if title is not None and title.text:
-                title_string = title.text
+                title_string = title.text.strip()
             else:
                 xml2rfc.log.warn('No title specified in reference', \
                                  reference.attrib.get('anchor', ''))
@@ -551,7 +569,8 @@ class HtmlRfcWriter(BaseRfcWriter):
                                table.attrib.get('align', self.defaults['table_align']) )
         cellpadding = '3'
         cellspacing = '0'
-        htmltable = E.TABLE(cellpadding=cellpadding, cellspacing=cellspacing)
+        htmltable = E.TABLE(cellpadding=cellpadding, )
+        htmltable.attrib['cellspacing'] = cellspacing
         htmltable.attrib['class'] = style
 
         # Add caption, if it exists
@@ -622,7 +641,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         for letter in sorted(alpha_bucket.keys()):
             # Add letter element
             table.append(E.TR(E.TD(E.STRONG(letter))))
-            for item in alpha_bucket[letter]:
+            for item in sorted(alpha_bucket[letter]):
                 # Add item element
                 anchor = self._iref_index[item].anchor or ''
                 if anchor:
@@ -631,7 +650,8 @@ class HtmlRfcWriter(BaseRfcWriter):
                 else:
                     td = E.TD(item)
                 table.append(E.TR(E.TD(' '), td))
-                for name, subitem in self._iref_index[item].subitems.items():
+                for name in sorted(self._iref_index[item].subitems.keys()):
+                    subitem = self._iref_index[item].subitems[name]
                     # Add subitem element
                     td = E.TD()
                     td.text = (u'\u00a0\u00a0')  # Spaces
@@ -670,8 +690,7 @@ class HtmlRfcWriter(BaseRfcWriter):
             description = abs_t.text
         keywords = self.r.findall('front/keyword')
         keyword_list = [keyword.text for keyword in keywords if keyword.text]
-        generator = "xml2rfc version %s - http://tools.ietf.org/tools/xml2rfc" % \
-                    '.'.join(map(str, xml2rfc.VERSION))
+        generator = "xml2rfc version %s - http://tools.ietf.org/tools/xml2rfc" % xml2rfc.__version__
 
         # Build ISO8601 date string
         docDate = ''
