@@ -6,6 +6,7 @@
 import datetime
 try:
     import debug
+    assert debug
 except ImportError:
     pass
 
@@ -129,8 +130,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
             year = date.attrib.get('year', '')
             self.right_header = month + ' ' + year
         authors = self.r.findall('front/author')
-        surnames = map(lambda author: author.attrib.get('surname', ''), authors)
-        surnames = filter(lambda surname: bool(surname), surnames)
+        surnames = [ a.attrib.get('surname') for a in authors if a.attrib.get('surname', None) ]
         if len(surnames) == 1:
             self.left_footer = surnames[0]
         elif len(surnames) == 2:
@@ -148,6 +148,13 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         self.left_header = self.pis.get('header', self.left_header)
 
     def page_break(self, final=False):
+        # remove header if nothing on the page
+        if final and self.page_length == 1:
+            while len(self.output) > 0:
+                p = self.output.pop()
+                if p == '\f':
+                    break;
+            return
         self.output.append('')
         self.output.append('')
         self.output.append('')
@@ -161,12 +168,16 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
     def emit(self, text):
         """Write text to the output buffer if it's not just a blank
            line at the top of the page"""
-        if self.page_length == 1 and text.strip() == '':
-            return 
-        if isinstance(text, basestring):
+        if isinstance(text, str):
+            if self.page_length == 1 and text.strip() == '':
+                return 
             self.output.append(text)
             self.page_length += 1
         elif isinstance(text, list):
+            if self.page_length == 1:
+                for line in text:
+                    self.emit(line)
+                return
             self.output.extend(text)
             self.page_length += len(text)
         else:
@@ -282,8 +293,9 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                         pages.append(self.page_num)
 
         # Write final footer
-        remainder = max_page_length - self.page_length - 2
-        self.emit([''] * remainder)
+        if self.page_length > 1:
+            remainder = max_page_length - self.page_length - 2
+            self.emit([''] * remainder)
         self.page_break(final=True)
         
         # Now we need to go back into the buffer and insert the real table 
