@@ -3,15 +3,13 @@
 # --------------------------------------------------
 
 # Python libs
-import os
 import textwrap
-import string
-import math
 import lxml
 import datetime
 import re
 try:
     import debug
+    assert debug
 except ImportError:
     pass
 
@@ -136,7 +134,17 @@ class RawTextRfcWriter(BaseRfcWriter):
         """ Writes a <list> element """
         bullet = '   '
         hangIndent = None
-        style = list.attrib.get('style', 'empty')
+        # style comes from the node if one exists
+        style = list.attrib.get('style', '')
+        if not style:
+            # otherwise look for the nearest list parent with a style and use it
+            for parent in list.iterancestors():
+                if parent.tag == 'list':
+                    style = parent.attrib.get('style', '')
+                    if style:
+                        break
+        if not style:
+            style = 'empty'
         listlength = len(list.findall('t'))        
         if style == 'hanging' or style.startswith('format'):
             # Check for optional hangIndent
@@ -148,7 +156,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         counter_index = None
         if style.startswith('format'):
             format_str = style.partition('format ')[2]
-            allowed_formats = ('%c', '%C', '%d', '%i', '%I')
+            allowed_formats = ('%c', '%C', '%d', '%i', '%I', '%o', '%x', '%X')
             if not any(map(lambda format: format in format_str, allowed_formats)):
                 xml2rfc.log.warn('Invalid format specified: %s ' 
                                  '(Must be one of %s)' % (style,
@@ -176,7 +184,8 @@ class RawTextRfcWriter(BaseRfcWriter):
                 elif style == 'numbers':
                     bullet = self._format_counter("%d.", t_count+1, listlength)
                 elif style == 'letters':
-                    bullet = self._format_counter("%c.", t_count+1, listlength)
+                    letter_style = "%C." if (level % 2) else "%c."
+                    bullet = self._format_counter(letter_style, t_count+1, listlength)
                 elif style == 'hanging':
                     bullet = element.attrib.get('hangText', '')
                     if len(bullet) < hangIndent:
@@ -189,7 +198,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     if bullet.endswith(':') and \
                     self.pis['colonspace'] == 'yes':
                         bullet+= ' '
-                    if element.text and len(bullet) > self.width/2:
+                    if element.text and len(bullet) > self.width//2:
                         # extra check of remaining space if the bullet is
                         # very long
                         first_word = self.wrapper._split(element.text)[0]
@@ -266,7 +275,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 if paging:
                     # Construct dots
                     dots = len(lines[-1]) % 2 and ' ' or ''
-                    dots += ' .' * int((self.width - len(lines[-1]) - len(dots) + 1)/2)
+                    dots += ' .' * int((self.width - len(lines[-1]) - len(dots) + 1)//2)
                     lines[-1] += dots
                     # Insert page
                     lines[-1] = lines[-1][:0 - len(pagestr)] + pagestr
@@ -291,12 +300,12 @@ class RawTextRfcWriter(BaseRfcWriter):
         for letter in sorted(alpha_bucket.keys()):
             # Write letter
             self.write_text(letter, indent=3, leading_blankline=True, buf=tmpbuf)
-            for item in alpha_bucket[letter]:
+            for item in sorted(alpha_bucket[letter]):
                 pages = self._iref_index[item].pages
                 # Write item
                 self.write_text(item + '  ' + ', '.join(map(str, pages))
                                                         , indent=6, buf=tmpbuf)
-                for subitem in self._iref_index[item].subitems:
+                for subitem in sorted(self._iref_index[item].subitems.keys()):
                     pages = self._iref_index[item].subitems[subitem].pages
                     # Write subitem
                     self.write_text(subitem + '  ' + ', '.join(map(str,pages))
@@ -463,7 +472,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             if align == 'center':
                 # Find the longest line, and use that as a fixed center.
                 longest_line = len(max(lines, key=len))
-                center_indent = indent + ((self.width - indent - longest_line) / 2)
+                center_indent = indent + ((self.width - indent - longest_line) // 2)
                 indent_str = center_indent > indent and ' ' * center_indent or \
                                                         ' ' * indent
                 for line in lines:
@@ -599,7 +608,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             lines.append(author.attrib.get('fullname', ''))
         organization = author.find('organization')
         if organization is not None and organization.text:
-            lines.append(organization.text)
+            lines.append(organization.text.strip())
         address = author.find('address')
         if address is not None:
             postal = address.find('postal')
@@ -655,7 +664,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             refstring.append(', ')
             title = ref.find('front/title')
             if title is not None and title.text:
-                refstring.append('"' + title.text + '", ')
+                refstring.append('"' + title.text.strip() + '", ')
             else:
                 xml2rfc.log.warn('No title specified in reference',
                                  ref.attrib.get('anchor', ''))
@@ -668,14 +677,17 @@ class RawTextRfcWriter(BaseRfcWriter):
             date = ref.find('front/date')
             if date is not None:
                 month = date.attrib.get('month', '')
-                if month:
-                    month += ' '
                 year = date.attrib.get('year', '')
-                refstring.append(month + year)
+                if month or year:
+                    if month:
+                        month += ' '
+                    refstring.append(month + year)
             # Target?
             target = ref.attrib.get('target')
             if target:
-                refstring.append(', <' + target + '>')
+                if not refstring[-1].endswith(', '):
+                    refstring.append(', ')
+                refstring.append('<' + target + '>')
             refstring.append('.')
             annotation = ref.find('annotation')
             # Use anchor or num depending on PI
@@ -778,10 +790,10 @@ class RawTextRfcWriter(BaseRfcWriter):
                         column_weights[i] = int_width / 100.0
                 except ValueError:
                     pass
-            spec_weights = filter(bool, column_weights)
+            spec_weights = [ c for c in column_weights if c ]
             if 0 < len(spec_weights) < num_columns:
                 # Use explicit weights and divvy remaining equally
-                avg = (1 - sum(spec_weights)) /  num_columns - len(spec_weights)
+                avg = (1 - sum(spec_weights)) //  num_columns - len(spec_weights)
                 for i, weight in enumerate(column_weights):
                     if not weight:
                         column_weights[i] = avg
@@ -790,10 +802,11 @@ class RawTextRfcWriter(BaseRfcWriter):
                 # each column as its minimum width.  If this sum exceeds max, cut
                 # each column from high to low until they all fit, and use those as
                 # weights.  Else, use longest_lines to fill in weights.
-                if sum(longest_words) > table_max_chars:
-                    column_weights = map(lambda x: float(x) / sum(longest_words), longest_words)
+                lwtot = sum(longest_words)
+                if lwtot > table_max_chars:
+                    column_weights = [ float(l)/lwtot for l in longest_words ]
                 else:
-                    column_weights = map(lambda x: float(x) / table_max_chars, longest_words)
+                    column_weights = [ float(l)/table_max_chars for l in longest_words ]
                     remainder = 1 - sum(column_weights)
                     for i, weight in enumerate(column_weights):
                         column_weights[i] += remainder * \
@@ -803,7 +816,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 pass
 
             # Compile column widths and correct floating point error
-            column_widths = map(lambda x: int(x * table_max_chars), column_weights)
+            column_widths = [ int(w*table_max_chars) for w in column_weights ]
             while(sum(column_widths) < table_max_chars):
                 broken = False
                 for i, wordlen in enumerate(longest_words):
@@ -847,6 +860,8 @@ class RawTextRfcWriter(BaseRfcWriter):
 
         # Draw the table
         for i, cell_line in enumerate(cell_lines):
+            if i==0 and cell_line==[['']]:
+                continue
             # produce as many outpur rows as the number of wrapped
             # text lines in the cell with most lines, but at least 1
             for row in range(0, max(map(len, cell_line))):
